#!/bin/bash

my_version=0.62

retry_every=60

conf=/usr/local/etc/selfddns-update.conf

me=$(basename "$0")

log() {
	if [ -t 1 ] ; then
		echo "$@"
	else
		logger -t "$me" "$@"
	fi
}

warn() {
	if [ -t 1 ] ; then
		echo "$@" 1>&2
	else
		logger -t "$me" "$@"
	fi
}

rx='([1-9]?[0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])'
function ip_valid() {
    if [[ "$1" =~ ^$rx\.$rx\.$rx\.$rx$ ]]; then
	return 0
    else
	return 1
    fi
}



sleep 5 # give some time for networking to come up

iface=$(route -n get 0.0.0.0 2>/dev/null | awk '/interface: / {print $2}')
if [ -z "$iface" ]; then
	warn "Error: no default route yet. Retrying in 30 seconds."
	sleep 30
	iface=$(route -n get 0.0.0.0 2>/dev/null | awk '/interface: / {print $2}')
fi
if [ -z "$iface" ]; then
	warn "Error: Still no default route. Quitting."
	exit 1
fi

[ ! -f "$conf" ] && warn "Error: config file $conf not found"

source "$conf" || { warn "Error $? trying to source $conf"; exit 1; }

for var in dns_server zone host secret ipsource; do
	v=${!var}
	if [ -z "$v" ]; then
		warn "Error: Undefined variable \$$var in $conf"
		exit 1
	else
		log "$var = $v"
	fi
done

# Set default TTL if it was not set in conf file
[ -z "$ttl" ] && ttl=3600

now=$(date "+%F-%Hh%M.%S")
ip=$(curl -qs $ipsource)

until ip_valid "$ip"; do
	warn "Error: could not get IP from $ipsource. Will retry in $retry_every seconds..."
	sleep $retry_every
	ip=$(curl -qs $ipsource)
done

name=$(dig +short -x $ip)

ap=$(networksetup -getairportnetwork en1 | perl -pe 's/^.*?:\s*//') || ap=''
log "$now. (v=$my_version. AP=$ap. Name=$name) $@"

update="server $dns_server
zone $zone.
key $host.$zone $secret
update delete $host.$zone.
update add $host.$zone. $ttl A $ip
update add $host.$zone. $ttl TXT \"Updated on $now. (v=$my_version. AP=$ap. Name=$name)\"
send
"

echo "$update" | nsupdate

rv=$?

if [ $rv -ne 0 ]; then 
	warn "Error $rv trying to update with '$update'"
	exit 1
else
	log "OK. $host.$zone updated to $ip"
fi
